@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005--2008  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -262,7 +262,7 @@ static dba_err encoder_encode_sec1ed3(encoder e)
 	/* Update sequence number (zero for original BUFR messages; incremented for updates) */
 	DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
 	/* Bit 1= 0 No optional section = 1 Optional section included Bits 2 ­ 8 set to zero (reserved) */
-	DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
+	DBA_RUN_OR_RETURN(encoder_append_byte(e, e->in->opt.bufr.optional_section_length ? 0x80 : 0));
 
 	/* Data category (BUFR Table A) */
 	/* Data sub-category (defined by local ADP centres) */
@@ -303,7 +303,7 @@ static dba_err encoder_encode_sec1ed4(encoder e)
 	/* Update sequence number (zero for original BUFR messages; incremented for updates) */
 	DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
 	/* Bit 1= 0 No optional section = 1 Optional section included Bits 2 ­ 8 set to zero (reserved) */
-	DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
+	DBA_RUN_OR_RETURN(encoder_append_byte(e, e->in->opt.bufr.optional_section_length ? 0x80 : 0));
 
 	/* Data category (BUFR Table A) */
 	DBA_RUN_OR_RETURN(encoder_append_byte(e, e->in->type));
@@ -372,6 +372,22 @@ dba_err bufr_encoder_encode(bufrex_msg in, dba_rawmsg out)
 
 	/* Encode BUFR section 2 (Optional section) */
 	/* Nothing to do */
+	if (e->in->opt.bufr.optional_section_length)
+	{
+		int pad;
+		/* Length of section */
+		if ((pad = (e->in->opt.bufr.optional_section_length % 2 == 1)))
+			DBA_RUN_OR_RETURN(encoder_add_bits(e, 4 + e->in->opt.bufr.optional_section_length + 1, 24));
+		else
+			DBA_RUN_OR_RETURN(encoder_add_bits(e, 4 + e->in->opt.bufr.optional_section_length, 24));
+		/* Set to 0 (reserved) */
+		DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
+
+		DBA_RUN_OR_RETURN(encoder_raw_append(e, e->in->opt.bufr.optional_section, e->in->opt.bufr.optional_section_length));
+		// Padd to even number of bytes
+		if (pad)
+			DBA_RUN_OR_RETURN(encoder_append_byte(e, 0));
+	}
 
 	TRACE("sec2 ends at %d\n", e->out->len);
 	e->sec3_start = e->out->len;
@@ -625,10 +641,12 @@ static dba_err encoder_encode_b_data(encoder e)
 		TRACE("Converted to int (ref %d, scale %d): %d\n", info->bit_ref, info->bufr_scale - e->c_scale_change, ival);
 		if (e->c_width_change != 0)
 			TRACE("Width change: %d\n", e->c_width_change);
-		TRACE("Writing with size %d\n", ival, len + e->c_width_change);
+		TRACE("Writing %u with size %d\n", ival, len + e->c_width_change);
 		/* In case of overflow, store 'missing value' */
 		if ((unsigned)ival >= (1u<<(len + e->c_width_change)))
 		{
+			err = dba_error_consistency("value %f does not fit in variable B%02d%03d", dval, DBA_VAR_X(info->var), DBA_VAR_Y(info->var));
+			goto cleanup;
 			TRACE("Overflow: %x %u %d >= (1<<(%u + %u)) = %x %u %d\n",
 				ival, ival, ival,
 				len, e->c_width_change,
