@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2009  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -427,6 +427,24 @@ static inline int normalon(int lon)
 	return ((lon + 18000000) % 36000000) - 18000000;
 }
 
+static dba_err add_repinfo_where(dba_db_cursor cur, dba_querybuf buf, dba_record query, const char* colname)
+{
+	const char* val;
+#define ADD_INT(key, sql, needed) do { \
+	if ((val = dba_record_key_peek_value(query, key)) != NULL) { \
+		int ival = strtol(val, 0, 10); \
+		/*TRACE("found %s: adding %s. val is %d\n", info(key)->desc, sql, *out);*/ \
+		DBA_RUN_OR_RETURN(dba_querybuf_append_listf(buf, sql, colname, ival)); \
+		cur->from_wanted |= needed; \
+	} } while (0)
+	
+	ADD_INT(DBA_KEY_PRIORITY, "%s.prio=%d", DBA_DB_FROM_RI);
+	ADD_INT(DBA_KEY_PRIOMIN, "%s.prio>=%d", DBA_DB_FROM_RI);
+	ADD_INT(DBA_KEY_PRIOMAX, "%s.prio<=%d", DBA_DB_FROM_RI);
+#undef ADD_INT
+	return dba_error_ok();
+}
+
 /*
  * Create the WHERE part of the query
  */
@@ -624,9 +642,7 @@ static dba_err make_where(dba_db_cursor cur, dba_record query)
 		cur->from_wanted |= DBA_DB_FROM_D;
 	}
 
-	ADD_INT(&cur->sel_priority, DBA_KEY_PRIORITY, "ri.prio=?", DBA_DB_FROM_RI);
-	ADD_INT(&cur->sel_priomin, DBA_KEY_PRIOMIN, "ri.prio>=?", DBA_DB_FROM_RI);
-	ADD_INT(&cur->sel_priomax, DBA_KEY_PRIOMAX, "ri.prio<=?", DBA_DB_FROM_RI);
+	DBA_RUN_OR_RETURN(add_repinfo_where(cur, cur->where, query, "ri"));
 
 	if ((val = dba_record_var_peek_value(query, DBA_VAR(0, 1, 1))) != NULL)
 	{
@@ -847,6 +863,7 @@ static dba_err add_other_froms(dba_db_cursor cur, unsigned int base)
 	return dba_error_ok();
 }
 
+#if 0
 static dba_err rowcount(dba_db db, const char* table, DBALLE_SQL_C_SINT_TYPE* count)
 {
 	dba_err err = DBA_OK;
@@ -889,6 +906,7 @@ static dba_err setstmtattr(SQLHSTMT stm, SQLINTEGER attr, SQLPOINTER val, SQLINT
 		return dba_db_error_odbc(SQL_HANDLE_STMT, stm, context);
 	return dba_error_ok();
 }
+#endif
 
 /* FIXME: this is a temporary solution giving an approximate row count only:
  * insert/delete/update queries run between the count and the select will
@@ -965,7 +983,11 @@ static dba_err getcount(dba_db_cursor cur, dba_record query, unsigned int wanted
 				break;
 			default:
 				DBA_RUN_OR_RETURN(dba_querybuf_append(cur->query,
-					" AND ri.prio=(SELECT MAX(sri.prio) FROM repinfo sri JOIN context sc ON sri.id=sc.id_report JOIN data sd ON sc.id=sd.id_context WHERE sc.id_ana=c.id_ana AND sc.ltype1=c.ltype1 AND sc.l1=c.l1 AND sc.ltype2=c.ltype2 AND sc.l2=c.l2 AND sc.ptype=c.ptype AND sc.p1=c.p1 AND sc.p2=c.p2 AND sc.datetime=c.datetime AND sd.id_var=d.id_var) "));
+					" AND ri.prio=(SELECT MAX(sri.prio) FROM repinfo sri JOIN context sc ON sri.id=sc.id_report JOIN data sd ON sc.id=sd.id_context WHERE "));
+				DBA_RUN_OR_RETURN(dba_querybuf_start_list(cur->query, " AND "));
+				DBA_RUN_OR_RETURN(add_repinfo_where(cur, cur->query, query, "sri"));
+				DBA_RUN_OR_RETURN(dba_querybuf_append_list(cur->query,
+					"sc.id_ana=c.id_ana AND sc.ltype1=c.ltype1 AND sc.l1=c.l1 AND sc.ltype2=c.ltype2 AND sc.l2=c.l2 AND sc.ptype=c.ptype AND sc.p1=c.p1 AND sc.p2=c.p2 AND sc.datetime=c.datetime AND sd.id_var=d.id_var) "));
 				break;
 		}
 
@@ -1092,7 +1114,11 @@ dba_err dba_db_cursor_query(dba_db_cursor cur, dba_record query, unsigned int wa
 				/* Continue to the query */
 			default:
 				DBA_RUN_OR_RETURN(dba_querybuf_append(cur->query,
-					" AND ri.prio=(SELECT MAX(sri.prio) FROM repinfo sri JOIN context sc ON sri.id=sc.id_report JOIN data sd ON sc.id=sd.id_context WHERE sc.id_ana=c.id_ana AND sc.ltype1=c.ltype1 AND sc.l1=c.l1 AND sc.ltype2=c.ltype2 AND sc.l2=c.l2 AND sc.ptype=c.ptype AND sc.p1=c.p1 AND sc.p2=c.p2 AND sc.datetime=c.datetime AND sd.id_var=d.id_var) "));
+					" AND ri.prio=(SELECT MAX(sri.prio) FROM repinfo sri JOIN context sc ON sri.id=sc.id_report JOIN data sd ON sc.id=sd.id_context WHERE "));
+				DBA_RUN_OR_RETURN(dba_querybuf_start_list(cur->query, " AND "));
+				DBA_RUN_OR_RETURN(add_repinfo_where(cur, cur->query, query, "sri"));
+				DBA_RUN_OR_RETURN(dba_querybuf_append_list(cur->query,
+					"sc.id_ana=c.id_ana AND sc.ltype1=c.ltype1 AND sc.l1=c.l1 AND sc.ltype2=c.ltype2 AND sc.l2=c.l2 AND sc.ptype=c.ptype AND sc.p1=c.p1 AND sc.p2=c.p2 AND sc.datetime=c.datetime AND sd.id_var=d.id_var) "));
 				break;
 		}
 
@@ -1158,9 +1184,9 @@ dba_err dba_db_cursor_query(dba_db_cursor cur, dba_record query, unsigned int wa
 	DBA_RUN_OR_RETURN(setstmtattr(cur->stm, SQL_ATTR_CURSOR_SCROLLABLE, (SQLPOINTER)SQL_SCROLLABLE, SQL_IS_INTEGER, "Setting SQL_SCROLLABLE"));
 	DBA_RUN_OR_RETURN(setstmtattr(cur->stm, SQL_ATTR_CURSOR_TYPE, (SQLPOINTER)SQL_CURSOR_DYNAMIC, SQL_IS_INTEGER, "Setting SQL_CURSOR_DYNAMIC"));
 
-	fprintf(stderr, "********************** 0 ************\n");
-	fprintf(stderr, "** Q %s\n", dba_querybuf_get(cur->query));
 #endif
+	//fprintf(stderr, "********************** 0 ************\n");
+	//fprintf(stderr, "** Q %s\n", dba_querybuf_get(cur->query));
 
 	/* Perform the query */
 	{
