@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,9 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h>
+#include <limits.h>
+#include <unistd.h>
 
 #include <assert.h>
 
@@ -72,7 +75,7 @@ static const char* init_functions[] = {
 static const char* init_queries_mysql[] = {
 	"CREATE TABLE repinfo ("
 	"   id		     SMALLINT PRIMARY KEY,"
-	"	memo	 	 VARCHAR(30) NOT NULL,"
+	"	memo	 	 VARCHAR(20) NOT NULL,"
 	"	description	 VARCHAR(255) NOT NULL,"
 	"   prio	     INTEGER NOT NULL,"
 	"	descriptor	 CHAR(6) NOT NULL,"
@@ -373,13 +376,11 @@ static dba_err dba_db_get_rep_cod(dba_db db, dba_record rec, int* id)
 	return dba_error_ok();
 }		
 
-
-dba_err dba_db_create(const char* dsn, const char* user, const char* password, dba_db* db)
+// First part of initialising a dba_db
+static dba_err dba_db_preinit(dba_db* db)
 {
 	dba_err err = DBA_OK;
-	char drivername[50];
 	int sqlres;
-	SQLSMALLINT len;
 
 	if (dba_od_env_initialized == 0)
 	{
@@ -419,18 +420,20 @@ dba_err dba_db_create(const char* dsn, const char* user, const char* password, d
 	/* Set the connection timeout */
 	/* SQLSetConnectAttr(pc.od_conn, SQL_LOGIN_TIMEOUT, (SQLPOINTER *)5, 0); */
 
-	/* Connect to the DSN */
-	sqlres = SQLConnect((*db)->od_conn,
-						(SQLCHAR*)dsn, SQL_NTS,
-						(SQLCHAR*)user, SQL_NTS,
-						(SQLCHAR*)(password == NULL ? "" : password), SQL_NTS);
-	if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
-	{
-		err = dba_db_error_odbc(SQL_HANDLE_DBC, (*db)->od_conn,
-				"Connecting to DSN %s as user %s", dsn, user);
-		goto fail;
-	}
-	(*db)->connected = 1;
+	return dba_error_ok();
+
+fail:
+	dba_db_delete(*db);
+	*db = 0;
+	return err;
+}
+
+static dba_err dba_db_postinit(dba_db* db)
+{
+	dba_err err = DBA_OK;
+	char drivername[50];
+	int sqlres;
+	SQLSMALLINT len;
 
 	/* Find out what kind of database we are working with */
 	sqlres = SQLGetInfo((*db)->od_conn, SQL_DRIVER_NAME, (SQLPOINTER)drivername, 50, &len);
@@ -456,11 +459,17 @@ dba_err dba_db_create(const char* dsn, const char* user, const char* password, d
 
 #ifdef DBA_USE_TRANSACTIONS
 	/* Set manual commit */
-	sqlres = SQLSetConnectAttr((*db)->od_conn, SQL_ATTR_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF, 0);
-	if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
+	if ((*db)->server_type != SQLITE)
 	{
-		err = dba_db_error_odbc(SQL_HANDLE_DBC, (*db)->od_conn, "Disabling ODBC autocommit");
-		goto fail;
+		sqlres = SQLSetConnectAttr((*db)->od_conn, SQL_ATTR_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF, 0);
+		if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
+		{
+			err = dba_db_error_odbc(SQL_HANDLE_DBC, (*db)->od_conn, "Disabling ODBC autocommit");
+			goto fail;
+		}
+	} else {
+		DBA_RUN_OR_GOTO(fail, dba_db_run_sql(*db, "PRAGMA journal_mode = MEMORY"));
+		DBA_RUN_OR_GOTO(fail, dba_db_run_sql(*db, "PRAGMA legacy_file_format = 0"));
 	}
 
     /* TODO: no need to precompile all these queries anymore */
@@ -504,18 +513,20 @@ dba_err dba_db_create(const char* dsn, const char* user, const char* password, d
 			DBA_RUN_OR_GOTO(fail, dba_db_seq_create((*db), "seq_pseudoana", &((*db)->seq_pseudoana)));
 			DBA_RUN_OR_GOTO(fail, dba_db_seq_create((*db), "seq_context", &((*db)->seq_context)));
 			break;
-		default:
+		case MYSQL:
 			DBA_RUN_OR_GOTO(fail, dba_db_statement_create(*db, &((*db)->stm_last_insert_id)));
 			SQLBindCol((*db)->stm_last_insert_id, 1, DBALLE_SQL_C_SINT, &((*db)->last_insert_id), sizeof((*db)->last_insert_id), 0);
-			switch ((*db)->server_type)
+			sqlres = SQLPrepare((*db)->stm_last_insert_id, (unsigned char*)"SELECT LAST_INSERT_ID()", SQL_NTS);
+			if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
 			{
-				case MYSQL:
-					sqlres = SQLPrepare((*db)->stm_last_insert_id, (unsigned char*)"SELECT LAST_INSERT_ID()", SQL_NTS);
-					break;
-				case SQLITE:
-					sqlres = SQLPrepare((*db)->stm_last_insert_id, (unsigned char*)"SELECT LAST_INSERT_ROWID()", SQL_NTS);
-					break;
+				err = dba_db_error_odbc(SQL_HANDLE_STMT, (*db)->stm_last_insert_id, "compiling query for querying the last insert id");
+				goto fail;
 			}
+			break;
+		case SQLITE:
+			DBA_RUN_OR_GOTO(fail, dba_db_statement_create(*db, &((*db)->stm_last_insert_id)));
+			SQLBindCol((*db)->stm_last_insert_id, 1, DBALLE_SQL_C_SINT, &((*db)->last_insert_id), sizeof((*db)->last_insert_id), 0);
+			sqlres = SQLPrepare((*db)->stm_last_insert_id, (unsigned char*)"SELECT LAST_INSERT_ROWID()", SQL_NTS);
 			if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
 			{
 				err = dba_db_error_odbc(SQL_HANDLE_STMT, (*db)->stm_last_insert_id, "compiling query for querying the last insert id");
@@ -532,6 +543,127 @@ fail:
 	return err;
 }
 
+dba_err dba_db_create(const char* dsn, const char* user, const char* password, dba_db* db)
+{
+	dba_err err = DBA_OK;
+	int sqlres;
+	*db = NULL;
+
+	DBA_RUN_OR_GOTO(fail, dba_db_preinit(db));
+
+	/* Connect to the DSN */
+	sqlres = SQLConnect((*db)->od_conn,
+						(SQLCHAR*)dsn, SQL_NTS,
+						(SQLCHAR*)user, SQL_NTS,
+						(SQLCHAR*)(password == NULL ? "" : password), SQL_NTS);
+	if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
+	{
+		err = dba_db_error_odbc(SQL_HANDLE_DBC, (*db)->od_conn,
+				"Connecting to DSN %s as user %s", dsn, user);
+		goto fail;
+	}
+	(*db)->connected = 1;
+
+	DBA_RUN_OR_GOTO(fail, dba_db_postinit(db));
+
+	return dba_error_ok();
+	
+fail:
+	if (*db != NULL) dba_db_delete(*db);
+	*db = 0;
+	return err;
+}
+
+dba_err dba_db_create_generic(const char* config, dba_db* db)
+{
+	dba_err err = DBA_OK;
+	char sdcout[1024];
+	int sqlres;
+	SQLSMALLINT outlen;
+	*db = NULL;
+
+	DBA_RUN_OR_GOTO(fail, dba_db_preinit(db));
+
+	/* Connect to the DSN */
+	sqlres = SQLDriverConnect((*db)->od_conn, NULL,
+					(SQLCHAR*)config, SQL_NTS,
+					(SQLCHAR*)sdcout, 1024, &outlen,
+					SQL_DRIVER_NOPROMPT);
+
+	if ((sqlres != SQL_SUCCESS) && (sqlres != SQL_SUCCESS_WITH_INFO))
+	{
+		err = dba_db_error_odbc(SQL_HANDLE_DBC, (*db)->od_conn,
+				"Connecting to DB using configuration %s", config);
+		goto fail;
+	}
+	(*db)->connected = 1;
+
+	DBA_RUN_OR_GOTO(fail, dba_db_postinit(db));
+
+	return dba_error_ok();
+	
+fail:
+	if (*db != NULL) dba_db_delete(*db);
+	*db = 0;
+	return err;
+}
+
+dba_err dba_db_create_from_file(const char* pathname, dba_db* db)
+{
+	// Access sqlite file directly
+	char buf[PATH_MAX];
+	if (pathname[0] != '/')
+	{
+		char cwd[PATH_MAX];
+		snprintf(buf, PATH_MAX, "Driver=SQLite3;Database=%s/%s;", getcwd(cwd, PATH_MAX), pathname);
+	}
+	else
+		snprintf(buf, PATH_MAX, "Driver=SQLite3;Database=%s;", pathname);
+	return dba_db_create_generic(buf, db);
+}
+
+dba_err dba_db_create_test(dba_db* db)
+{
+	const char* envurl = getenv("DBA_DB");
+	if (envurl != NULL)
+		return dba_db_create_from_url(envurl, db);
+	else
+		return dba_db_create_from_file("test.sqlite", db);
+}
+
+dba_err dba_db_create_from_url(const char* url, dba_db* db)
+{
+	if (strncmp(url, "sqlite://", 9) == 0)
+		return dba_db_create_from_file(url + 9, db);
+	if (strncmp(url, "sqlite:", 7) == 0)
+		return dba_db_create_from_file(url + 7, db);
+	if (strncmp(url, "odbc://", 7) == 0)
+	{
+		char buf[PATH_MAX];
+		strncpy(buf, url + 7, PATH_MAX - 1);
+		buf[PATH_MAX - 1] = 0;
+		char* dsn = strchr(buf, '@');
+		if (dsn == NULL) return dba_db_create(buf, "", "", db); // odbc://dsn
+		// Split the string at '@'
+		*dsn = 0; ++dsn;
+		char* pwd = strchr(buf, ':');
+		if (pwd == NULL) return dba_db_create(dsn, buf, "", db); // odbc://user@dsn
+		*pwd = 0; ++pwd;
+		return dba_db_create(dsn, buf, pwd, db); // odbc://user:pass@dsn
+	}
+	if (strncmp(url, "test:", 5) == 0)
+		return dba_db_create_test(db);
+	return dba_error_consistency("unknown url \"%s\"", url);
+}
+
+int dba_db_is_url(const char* str)
+{
+	if (strncmp(str, "sqlite:", 7) == 0) return 1;
+	if (strncmp(str, "odbc://", 7) == 0) return 1;
+	if (strncmp(str, "test:", 5) == 0) return 1;
+	return 0;
+}
+
 void dba_db_delete(dba_db db)
 {
 	assert(db);
@@ -805,6 +937,15 @@ dba_err dba_db_rep_cod_from_memo(dba_db db, const char* memo, int* rep_cod)
 	return dba_db_repinfo_get_id(db->repinfo, memo, rep_cod);
 }
 
+dba_err dba_db_rep_memo_from_cod(dba_db db, int rep_cod, const char** memo)
+{
+	DBA_RUN_OR_RETURN(dba_db_need_repinfo(db));
+	dba_db_repinfo_cache c = dba_db_repinfo_get_by_id(db->repinfo, rep_cod);
+	if (c == NULL) return dba_error_notfound("looking for rep_memo corresponding to rep_cod '%d'", rep_cod);
+	*memo = c->memo;
+	return dba_error_ok();
+}
+
 dba_err dba_db_check_rep_cod(dba_db db, int rep_cod, int* valid)
 {
 	DBA_RUN_OR_RETURN(dba_db_need_repinfo(db));
