@@ -30,22 +30,7 @@ namespace tut_dballe {
 
 db_test::db_test() : db(NULL)
 {
-/*
-	const char* uname = getenv("DBA_USER");
-	if (uname == NULL)
-	{
-		struct passwd *pwd = getpwuid(getuid());
-		uname = pwd == NULL ? "test" : pwd->pw_name;
-	}
-*/
-	const char* dsn = getenv("DBA_TEST_DSN");
-	const char* user = getenv("DBA_TEST_USER");
-	const char* pass = getenv("DBA_TEST_PASS");
-	if (dsn == NULL) return;
-	if (user == NULL) user = "";
-	if (pass == NULL) pass = "";
-
-	CHECKED(dba_db_create(dsn, user, pass, &db));
+	CHECKED(dba_db_create_test(&db));
 	CHECKED(dba_db_reset(db, NULL));
 }
 db_test::~db_test()
