@@ -28,18 +28,17 @@
 namespace tut {
 using namespace tut_dballe;
 
-struct data_shar
+struct data_shar : public db_test
 {
 	TestMsgEnv testenv;
 
 	// DB handle
-	dba_db db;
 	dba_db_data da;
 
-	data_shar() : db(NULL)
+	data_shar()
 	{
-		CHECKED(create_dba_db(&db));
-		CHECKED(dba_db_reset(db, NULL));
+		if (!has_db()) return;
+
 		CHECKED(dba_db_need_data(db));
 		da = db->data;
 		gen_ensure(da != NULL);
@@ -95,12 +94,6 @@ struct data_shar
 		CHECKED(dba_db_context_insert(co, &id));
 		gen_ensure_equals(id, 2);
 	}
-
-	~data_shar()
-	{
-		CHECKED(dba_db_commit(db));
-		if (db != NULL) dba_db_delete(db);
-	}
 };
 TESTGRP(data);
 
@@ -108,6 +101,8 @@ TESTGRP(data);
 template<> template<>
 void to::test<1>()
 {
+	use_db();
+
 	// Test dba_db_data_set
 	dba_var var;
 	CHECKED(dba_var_create_local(DBA_VAR(0, 1, 2), &var));
@@ -128,46 +123,65 @@ void to::test<1>()
 template<> template<>
 void to::test<2>()
 {
+	use_db();
+
 	// Insert a datum
 	da->id_context = 1;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "123");
-	CHECKED(dba_db_data_insert(da, 0));
+	CHECKED(dba_db_data_insert_or_fail(da));
 	CHECKED(dba_db_commit(db));
 
 	// Insert another datum
 	da->id_context = 2;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "234");
-	CHECKED(dba_db_data_insert(da, 0));
+	CHECKED(dba_db_data_insert_or_fail(da));
 	CHECKED(dba_db_commit(db));
 
 	// Reinsert a datum: it should fail
 	da->id_context = 1;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "123");
-	gen_ensure_equals(dba_db_data_insert(da, 0), DBA_ERROR);
+	gen_ensure_equals(dba_db_data_insert_or_fail(da), DBA_ERROR);
 	CHECKED(dba_db_commit(db));
 
 	// Reinsert the other datum: it should fail
 	da->id_context = 2;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "234");
-	gen_ensure_equals(dba_db_data_insert(da, 0), DBA_ERROR);
+	gen_ensure_equals(dba_db_data_insert_or_fail(da), DBA_ERROR);
 	CHECKED(dba_db_commit(db));
 
 	// Reinsert a datum with overwrite: it should work
 	da->id_context = 1;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "123");
-	CHECKED(dba_db_data_insert(da, 1));
+	CHECKED(dba_db_data_insert_or_overwrite(da));
 	CHECKED(dba_db_commit(db));
 
 	// Reinsert the other datum with overwrite: it should work
 	da->id_context = 2;
 	da->id_var = DBA_VAR(0, 1, 2);
 	dba_db_data_set_value(da, "234");
-	CHECKED(dba_db_data_insert(da, 1));
+	CHECKED(dba_db_data_insert_or_overwrite(da));
+	CHECKED(dba_db_commit(db));
+
+	// Insert a new datum with ignore: it should insert
+	int inserted;
+	da->id_context = 3;
+	da->id_var = DBA_VAR(0, 1, 2);
+	dba_db_data_set_value(da, "234");
+	CHECKED(dba_db_data_insert_or_ignore(da, &inserted));
+	gen_ensure_equals(inserted, 1);
+	CHECKED(dba_db_commit(db));
+
+	// Reinsert the same datum with ignore: it should ignore
+	da->id_context = 3;
+	da->id_var = DBA_VAR(0, 1, 2);
+	dba_db_data_set_value(da, "234");
+	CHECKED(dba_db_data_insert_or_ignore(da, &inserted));
+	gen_ensure_equals(inserted, 0);
 	CHECKED(dba_db_commit(db));
 
 #if 0
