@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2008  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,12 @@
 dba_err aof_read_flight(const uint32_t* obs, int obs_len, dba_msg msg)
 {
 	int ltype = -1, l1 = -1;
+	int is_fixed = 0;
+	int flags_start = 25;
+
+	/* If the length is 28, then it's a new style message with dew point */
+	is_fixed = obs_len == 28;
+	if (is_fixed) flags_start = 26;
 
 	/* 07 Code type */
 	switch (OBS(7))
@@ -56,13 +62,44 @@ dba_err aof_read_flight(const uint32_t* obs, int obs_len, dba_msg msg)
 			DBA_RUN_OR_RETURN(dba_msg_set_flight_phase(msg, angle, -1));
 	}
 
-	if (OBS(20) != AOF_UNDEF)
+	/* 
+	 * If both pressure and height are present, use height, as makeaof makes
+	 * pressure a function of height
+	 */
+	if (OBS(24) != AOF_UNDEF)
 	{
+		/* Save the height in an analogous height layer.
+		 * We cannot save in the ana layer because a flight can pass twice in
+		 * the same point, at two different heights */
+
+		if (OBS(24) > AOF_UNDEF)
+		{
+			// The AOF specification says nothing about negative heights,
+			// completely disregarding the feelings of people in Schiphol or
+			// Furnace Creek.  Makeaof, however, happily encodes negative
+			// heights, so we try to match what it does.
+			// makeaof encodes negative numbers with whatever is used in the
+			// local machine.  We hope it is two's complement.
+			uint32_t absheight = ~OBS(24) + 1;
+			DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0,  7,  1), -(double)absheight, get_conf6((OBS(flags_start + 1) >> 18) & 0x3f),
+				102, OBS(24), 0, 0, 254, 0, 0));
+		} else {
+			DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0,  7,  1), (double)OBS(24), get_conf6((OBS(flags_start + 1) >> 18) & 0x3f),
+				102, OBS(24), 0, 0, 254, 0, 0));
+		}
+		if (ltype == -1)
+		{
+			ltype = 102;
+			l1 = OBS(24);
+		}
+	} else if (OBS(20) != AOF_UNDEF) {
 		double press = OBS(20) * 10.0;
 
-		/* Save the pressure in the anagraphical layer only */
-		/* FIXME: vedere se va memorizzato o no nell'anagrafica */
-		DBA_RUN_OR_RETURN(dba_msg_set_flight_press(msg, press, get_conf6(OBS(25) & 0x3f)));
+		/* Save the pressure in an analogous pressure layer.
+		 * We cannot save in the ana layer because a flight can pass twice in
+		 * the same point, at two different pressures */
+		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 10,  4), press, get_conf6(OBS(flags_start) & 0x3f),
+			100, press, 0, 0, 254, 0, 0));
 
 		/* Default to height for AMDAR */
 		if (msg->type != MSG_AMDAR || (OBS(24) == AOF_UNDEF))
@@ -72,17 +109,6 @@ dba_err aof_read_flight(const uint32_t* obs, int obs_len, dba_msg msg)
 			/* fprintf(stderr, "Press float: %f int: %d encoded: %s\n", press, l1, dba_var_value(dba_msg_get_flight_press_var(msg))); */
 		}
 	}
-	if (OBS(24) != AOF_UNDEF)
-	{
-		/* FIXME: vedere se va memorizzato o no nell'anagrafica (può non aver
-		 * senso se è calcolata dalla pressione) */
-		DBA_RUN_OR_RETURN(dba_msg_set_height(msg, (double)OBS(24), get_conf6((OBS(26) >> 18) & 0x3f)));
-		if (ltype == -1)
-		{
-			ltype = 102;
-			l1 = OBS(24);
-		}
-	}
 	if (ltype == -1)
 		return dba_error_notfound("looking for pressure or height in an AOF flight message");
 
@@ -90,13 +116,20 @@ dba_err aof_read_flight(const uint32_t* obs, int obs_len, dba_msg msg)
 
 	/* 21 Wind direction [degrees] */
 	if (OBS(21) != AOF_UNDEF)
-		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 11,  1), OBS(21), get_conf6(OBS(26) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
+		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 11,  1), OBS(21), get_conf6(OBS(flags_start + 1) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
 	/* 22 Wind speed [m/s] */
 	if (OBS(22) != AOF_UNDEF)
-		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 11,  2), OBS(22), get_conf6((OBS(26) >> 6) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
+		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 11,  2), OBS(22), get_conf6((OBS(flags_start + 1) >> 6) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
 	/* 23 Air temperature [1/10 K] */
 	if (OBS(23) != AOF_UNDEF)
-		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 12,  1), totemp(OBS(23)), get_conf6((OBS(26) >> 12) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
+		DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 12,  1), totemp(OBS(23)), get_conf6((OBS(flags_start + 1) >> 12) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
+
+	if (is_fixed)
+	{
+		if (OBS(25) != AOF_UNDEF)
+			DBA_RUN_OR_RETURN(dba_msg_setd(msg, DBA_VAR(0, 12,  3), totemp(OBS(23)), get_conf6((OBS(flags_start + 1) >> 24) & 0x3f), ltype, l1, 0, 0, 254, 0, 0));
+		
+	}
 
 	return dba_error_ok();
 }
