@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  */
 
 #include "exporters.h"
+#include "dballe/msg/context.h"
 
 static dba_err exporter101(dba_msg msg, bufrex_msg bmsg, bufrex_subset dst, int type);
 static dba_err exporter102(dba_msg msg, bufrex_msg bmsg, bufrex_subset dst, int type);
@@ -116,10 +117,10 @@ static dba_err run_template(dba_msg msg, bufrex_subset dst, struct template* tpl
 		}
 		else
 		{
-			dba_msg_datum d = dba_msg_find_by_id(msg, tpl[i].var);
+			dba_var var = dba_msg_find_by_id(msg, tpl[i].var);
 
-			if (d != NULL)
-				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, tpl[i].code, d->var));
+			if (var != NULL)
+				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, tpl[i].code, var));
 			else
 				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_undef(dst, tpl[i].code));
 		}
@@ -134,8 +135,7 @@ static int count_levels(dba_msg msg)
 
 	/* Count the number of levels */
 	for (i = 0; i < msg->data_count; i++)
-		if (msg->data[i]->ltype1 == 100 &&
-				dba_msg_level_find(msg->data[i], DBA_VAR(0, 8, 1), 254, 0, 0) != NULL)
+		if (dba_msg_context_find_vsig(msg->data[i]) != NULL)
 			lev_no++;
 
 	return lev_no;
@@ -147,25 +147,30 @@ static dba_err add_sounding_levels(dba_msg msg, bufrex_subset dst, dba_varcode*
 	/* Iterate backwards as we need to add levels in decreasing pressure order */
 	for (i = msg->data_count - 1; i >= 0; --i)
 	{
-		dba_msg_level lev = msg->data[i];
-		double press = lev->l1;
-		dba_msg_datum d;
-		dba_msg_datum p;
+		dba_msg_context ctx = msg->data[i];
+		double press = ctx->l1;
+		dba_var vss = dba_msg_context_find_vsig(ctx);
+		dba_var var;
 		int j;
 
-		if (lev->ltype1 != 100 ||
-				(d = dba_msg_level_find(lev, DBA_VAR(0, 8, 1), 254, 0, 0)) == NULL)
-			continue;
+		/* We only want levels with a vertical sounding significance */
+		if (vss == NULL) continue;
 
-		if ((p = dba_msg_level_find(lev, DBA_VAR(0, 10, 4), 254, 0, 0)) != NULL)
-			DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, DBA_VAR(0,  7,  4), p->var));
+		/* We only want pressure levels */
+		if (ctx->ltype1 != 100) continue;
+
+		/* Add pressure */
+		if ((var = dba_msg_context_find(ctx, DBA_VAR(0, 10, 4))) != NULL)
+			DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, DBA_VAR(0,  7,  4), var));
 		else
 			DBA_RUN_OR_RETURN(bufrex_subset_store_variable_d(dst, DBA_VAR(0,  7,  4), press));
-		DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, DBA_VAR(0,  8,  1), d->var));
+		/* Add vertical sounding significance */
+		DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, DBA_VAR(0,  8,  1), vss));
 
+		/* Add the rest */
 		for (j = 0; j < tpl_count; j++)
-			if ((d = dba_msg_level_find(lev, tpl[j], 254, 0, 0)) != NULL)
-				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, tpl[j], d->var));
+			if ((var = dba_msg_context_find(ctx, tpl[j])) != NULL)
+				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_var(dst, tpl[j], var));
 			else
 				DBA_RUN_OR_RETURN(bufrex_subset_store_variable_undef(dst, tpl[j]));
 	}
