@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #ifndef CONVERSION_H
 #define CONVERSION_H
 
+#include <dballe/core/file.h>
 #include <dballe/core/rawmsg.h>
 #include <dballe/msg/msgs.h>
 #include <dballe/bufrex/msg.h>
@@ -29,9 +30,16 @@
 
 struct conversion_info
 {
+	dba_file file;
+	int dest_type;
+	int dest_subtype;
+	int dest_localsubtype;
+	const char* dest_rep_memo;
+#if 0
 	dba_encoding outType;
 	void* outAction;
 	void* outActionData;
+#endif
 };
 
 dba_err convert_message(dba_rawmsg msg, bufrex_msg braw, dba_msgs decoded, void* data);
