@@ -24,8 +24,11 @@
 
 #include <dballe/core/record.h>
 #include <dballe/db/db.h>
+#include "simple.h"
 #include <assert.h>
 
+extern "C" {
+
 #define FDBA_HANDLE_START_DECL(name) \
 	struct fdba_handle_##name { \
 		int used;
@@ -81,15 +84,7 @@ FDBA_HANDLE_END_DECL(session)
 
 FDBA_HANDLE_START_DECL(simple)
 	int session;
-	int perms;
-	dba_record input;
-	dba_record output;
-	dba_record qcinput;
-	dba_record qcoutput;
-	dba_db_cursor ana_cur;
-	dba_db_cursor query_cur;
-	dba_record_cursor qc_iter;
-	int qc_count;
+	dballef::API* api;
 FDBA_HANDLE_END_DECL(simple)
 
 typedef void (*fdba_error_callback)(INTEGER(data));
@@ -100,4 +95,6 @@ FDBA_HANDLE_START_DECL(errcb)
 	int data;
 FDBA_HANDLE_END_DECL(errcb)
 
+}
+
 #endif
