@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ struct poptOption grepTable[] = {
 	POPT_TABLEEND
 };
 
-static char* op_dsn = "test";
+static char* op_dsn = "";
 static char* op_user = "";
 static char* op_pass = "";
 static char* op_input_type = "auto";
@@ -70,29 +70,39 @@ int op_verbose = 0;
 
 struct poptOption dbTable[] = {
 	{ "dsn", 0, POPT_ARG_STRING, &op_dsn, 0,
-		"DSN to use for connecting to the DBALLE database" },
+		"DSN, or URL-like database definition, to use for connecting to the DB-All.e database (can also be specified in the environment as DBA_DB)" },
 	{ "user", 0, POPT_ARG_STRING, &op_user, 0,
-		"username to use for connecting to the DBALLE database" },
+		"username to use for connecting to the DB-All.e database" },
 	{ "pass", 0, POPT_ARG_STRING, &op_pass, 0,
-		"password to use for connecting to the DBALLE database" },
+		"password to use for connecting to the DB-All.e database" },
 	POPT_TABLEEND
 };
 
 static dba_err create_dba_db(dba_db* db)
 {
-	if (op_user[0] == 0)
+	const char* chosen_dsn;
+
+	/* If dsn is missing, look in the environment */
+	if (op_dsn[0] == 0)
 	{
-		struct passwd *pwd = getpwuid(getuid());
-		op_user = pwd == NULL ? "test" : pwd->pw_name;
-	}
-	return dba_db_create(op_dsn, op_user, op_pass, db);
+		chosen_dsn = getenv("DBA_DB");
+		if (chosen_dsn == NULL)
+			return dba_error_consistency("no database specified");
+	} else
+		chosen_dsn = op_dsn;
+
+	/* If dsn looks like a url, treat it accordingly */
+	if (dba_db_is_url(chosen_dsn))
+		return dba_db_create_from_url(chosen_dsn, db);
+	else
+		return dba_db_create(chosen_dsn, op_user, op_pass, db);
 }
 
 struct import_data
 {
 	dba_db db;
 	int overwrite;
-	int forced_repcod;
+	const char* forced_repmemo;
 };
 
 static dba_err import_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs, void* data)
@@ -117,14 +127,14 @@ static dba_err import_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs, v
 	for (i = 0; i < msgs->len; ++i)
 	{
 		dba_msg msg = msgs->msgs[i];
-		if (d->forced_repcod == -1 && msg->type == MSG_GENERIC)
+		if (d->forced_repmemo == NULL && msg->type == MSG_GENERIC)
 		{
 			/* Put generic messages in the generic rep_cod by default */
-			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, 255, import_flags));
+			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, NULL, import_flags));
 		}
 		else
 		{
-			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, d->forced_repcod, import_flags));
+			DBA_RUN_OR_RETURN(dba_import_msg(d->db, msg, d->forced_repmemo, import_flags));
 		}
 	}
 	return dba_error_ok();
@@ -270,7 +280,7 @@ dba_err do_repinfo(poptContext optCon)
 	return dba_error_ok();
 }
 
-dba_err parse_op_report(dba_db db, int* res)
+dba_err parse_op_report(dba_db db, const char** res)
 {
 	if (op_report[0] != 0)
 	{
@@ -281,21 +291,14 @@ dba_err parse_op_report(dba_db db, int* res)
 				is_cod = 0;
 		
 		if (is_cod)
-		{
-			int valid;
-			*res = strtoul(op_report, NULL, 0);
-			DBA_RUN_OR_RETURN(dba_db_check_rep_cod(db, *res, &valid));
-			if (valid)
-				return dba_error_ok();
-			else
-				return dba_error_consistency("report code %d not found in the database", *res);
-		}
+			return dba_db_rep_memo_from_cod(db, strtoul(op_report, NULL, 0), res);
 		else
 		{
-			return dba_db_rep_cod_from_memo(db, op_report, res);
+			*res = op_report;
+			return dba_error_ok();
 		}
 	} else {
-		*res = -1;
+		*res = NULL;
 		return dba_error_ok();
 	}
 }
@@ -312,7 +315,7 @@ dba_err do_import(poptContext optCon)
 
 	DBA_RUN_OR_RETURN(create_dba_db(&data.db));
 	data.overwrite = op_overwrite;
-	DBA_RUN_OR_RETURN(parse_op_report(data.db, &(data.forced_repcod)));
+	DBA_RUN_OR_RETURN(parse_op_report(data.db, &(data.forced_repmemo)));
 
 	DBA_RUN_OR_RETURN(process_all(optCon, type, &grepdata, import_message, (void*)&data));
 
@@ -327,18 +330,18 @@ struct export_data
 	int cat;
 	int subcat;
 	int localsubcat;
-	int forced_rep_cod;
+	const char* forced_rep_memo;
 };
 
 static dba_err msg_writer(dba_msgs msgs, void* data)
 {
 	struct export_data* d = (struct export_data*)data;
 	/* Override the message type if the user asks for it */
-	if (d->forced_rep_cod != -1)
+	if (d->forced_rep_memo != NULL)
 	{
 		int i;
 		for (i = 0; i < msgs->len; ++i)
-			msgs->msgs[i]->type = dba_msg_type_from_repcod(d->forced_rep_cod);
+			msgs->msgs[i]->type = dba_msg_type_from_repmemo(d->forced_rep_memo);
 	}
 	DBA_RUN_OR_RETURN(dba_file_write_msgs(d->file, msgs, d->cat, d->subcat, d->localsubcat));
 	dba_msgs_delete(msgs);
@@ -355,7 +358,7 @@ static dba_err msg_dumper(dba_msgs msgs, void* data)
 
 dba_err do_export(poptContext optCon)
 {
-	struct export_data d = { NULL, 0, 0, 0, -1 };
+	struct export_data d = { NULL, 0, 0, 0, NULL };
 	dba_encoding type;
 	dba_record query;
 	dba_db db;
@@ -379,7 +382,7 @@ dba_err do_export(poptContext optCon)
 	{
 		DBA_RUN_OR_RETURN(dba_db_export(db, query, msg_dumper, stdout));
 	} else {
-		DBA_RUN_OR_RETURN(parse_op_report(db, &(d.forced_rep_cod)));
+		DBA_RUN_OR_RETURN(parse_op_report(db, &(d.forced_rep_memo)));
 
 		type = dba_cmdline_stringToMsgType(op_output_type, optCon);
 		DBA_RUN_OR_RETURN(dba_file_create(type, "(stdout)", "w", &d.file));
