@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,8 @@ static int op_dump_interpreted = 0;
 static int op_dump_text = 0;
 static char* op_input_type = "auto";
 static char* op_output_type = "bufr";
+static char* op_output_template = "";
+static char* op_report = "";
 static char* op_bisect_cmd = NULL;
 int op_verbose = 0;
 
@@ -82,6 +84,75 @@ static int count_nonnulls(bufrex_subset raw)
 	return count;
 }
 
+static dba_err dump_bufr_header(dba_rawmsg rmsg, bufrex_msg braw)
+{
+	int size;
+	const char *table_id;
+	const unsigned char *buf;
+
+	DBA_RUN_OR_RETURN(dba_rawmsg_get_raw(rmsg, &buf, &size));
+	DBA_RUN_OR_RETURN(bufrex_msg_get_table_id(braw, &table_id));
+
+	printf("Message %d\n", rmsg->index);
+	printf("Size: %d\n", size);
+	printf("Edition: %d\n", braw->edition);
+	printf("Centre: %d:%d\n", braw->opt.bufr.centre, braw->opt.bufr.subcentre);
+	printf("Category: %d:%d:%d\n", braw->type, braw->subtype, braw->localsubtype);
+	printf("Datetime: %04d-%02d-%02d %02d:%02d:%02d\n",
+			braw->rep_year, braw->rep_month, braw->rep_day,
+			braw->rep_hour, braw->rep_minute, braw->rep_second);
+	printf("Tables: %d:%d\n", braw->opt.bufr.master_table, braw->opt.bufr.local_table);
+	printf("Table: %s\n", table_id);
+	printf("Compression: %s\n", braw->opt.bufr.compression ? "yes" : "no");
+	printf("Optional section length: %d\n", braw->opt.bufr.optional_section_length);
+	printf("Subsets: %d (decoded %zd)\n\n", braw->opt.bufr.subsets, braw->subsets_count);
+
+	// Copy data descriptor section
+	//for (bufrex_opcode i = orig->datadesc; i != NULL; i = i->next)
+		//DBA_RUN_OR_GOTO(cleanup, bufrex_msg_append_datadesc(msg, i->val));
+
+	return dba_error_ok();
+}
+
+static dba_err dump_crex_header(dba_rawmsg rmsg, bufrex_msg braw)
+{
+	int size/*, checkdigit*/;
+	const char *table_id;
+	const unsigned char *buf;
+
+	DBA_RUN_OR_RETURN(dba_rawmsg_get_raw(rmsg, &buf, &size));
+	DBA_RUN_OR_RETURN(bufrex_msg_get_table_id(braw, &table_id));
+
+	printf("Message %d\n", rmsg->index);
+	printf("Size: %d\n", size);
+	printf("Edition: %d\n", braw->edition);
+	printf("Category: %d:%d:%d\n", braw->type, braw->subtype, braw->localsubtype);
+	printf("Datetime: %04d-%02d-%02d %02d:%02d:%02d\n",
+			braw->rep_year, braw->rep_month, braw->rep_day,
+			braw->rep_hour, braw->rep_minute, braw->rep_second);
+	printf("Tables: %d:%d\n", braw->opt.crex.master_table, braw->opt.crex.table);
+	printf("Table: %s\n", table_id);
+	printf("Check digit: %s\n\n", braw->opt.crex.has_check_digit ? "yes" : "no");
+
+	return dba_error_ok();
+}
+
+static dba_err dump_aof_header(dba_rawmsg rmsg)
+{
+	int size, category, subcategory;
+	const unsigned char *buf;
+
+	DBA_RUN_OR_RETURN(dba_rawmsg_get_raw(rmsg, &buf, &size));
+	DBA_RUN_OR_RETURN(aof_codec_get_category(rmsg, &category, &subcategory));
+	/* DBA_RUN_OR_RETURN(bufrex_message_get_vars(msg, &vars, &count)); */
+
+	printf("Message %d\n", rmsg->index);
+	printf("Size: %d\n", size);
+	printf("Category: %d:%d\n\n", category, subcategory);
+
+	return dba_error_ok();
+}
+
 static dba_err print_bufr_header(dba_rawmsg rmsg, bufrex_msg braw)
 {
 	int size;
@@ -159,6 +230,25 @@ static dba_err summarise_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs
 	return dba_error_ok();
 }
 
+static dba_err head_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs, void* data)
+{
+	switch (rmsg->encoding)
+	{
+		case BUFR:
+			if (braw == NULL) return dba_error_ok();
+			DBA_RUN_OR_RETURN(dump_bufr_header(rmsg, braw));
+			break;
+		case CREX:
+			if (braw == NULL) return dba_error_ok();
+			DBA_RUN_OR_RETURN(dump_crex_header(rmsg, braw));
+			break;
+		case AOF:
+			DBA_RUN_OR_RETURN(dump_aof_header(rmsg));
+			break;
+	}
+	return dba_error_ok();
+}
+
 static dba_err dump_dba_vars(bufrex_subset msg)
 {
 	int i;
@@ -313,6 +403,18 @@ dba_err do_scan(poptContext optCon)
 	return dba_error_ok();
 }
 
+dba_err do_head(poptContext optCon)
+{
+	/* Throw away the command name */
+	poptGetArg(optCon);
+
+	DBA_RUN_OR_RETURN(process_all(optCon, 
+				dba_cmdline_stringToMsgType(op_input_type, optCon),
+				&grepdata, head_message, 0));
+
+	return dba_error_ok();
+}
+
 dba_err do_dump(poptContext optCon)
 {
 	action action = op_dump_interpreted ? dump_cooked_message :
@@ -490,6 +592,7 @@ dba_err do_convert(poptContext optCon)
 {
 	dba_file file;
 	dba_encoding intype, outtype;
+	struct conversion_info convinfo;
 
 	/* Throw away the command name */
 	poptGetArg(optCon);
@@ -497,10 +600,26 @@ dba_err do_convert(poptContext optCon)
 	intype = dba_cmdline_stringToMsgType(op_input_type, optCon);
 	outtype = dba_cmdline_stringToMsgType(op_output_type, optCon);
 
+	if (op_report[0] != 0)
+		convinfo.dest_rep_memo = op_report;
+	else
+		convinfo.dest_rep_memo = NULL;
+
+	if (op_output_template[0] != 0)
+		DBA_RUN_OR_RETURN(bufrex_msg_parse_template(op_output_template,
+					&convinfo.dest_type, &convinfo.dest_subtype, &convinfo.dest_localsubtype));
+	else {
+		convinfo.dest_type = 0;
+		convinfo.dest_subtype = 0;
+		convinfo.dest_localsubtype = 0;
+	}
+
 	DBA_RUN_OR_RETURN(dba_file_create(outtype, "(stdout)", "w", &file));
 	/* DBA_RUN_OR_RETURN(dba_file_write_header(file, 0, 0)); */
 
-	DBA_RUN_OR_RETURN(process_all(optCon, intype, &grepdata, convert_message, (void*)file));
+	convinfo.file = file;
+
+	DBA_RUN_OR_RETURN(process_all(optCon, intype, &grepdata, convert_message, (void*)&convinfo));
 
 	dba_file_delete(file);
 
@@ -568,12 +687,12 @@ struct filter_data
 	int mindate[6];
 	int maxdate[6];
 	double latmin, latmax, lonmin, lonmax;
+	dba_var varfilter[20];
 };
 
 inline static int get_date_value(dba_msg msg, int value)
 {
-	dba_msg_datum d = dba_msg_find_by_id(msg, value);
-	dba_var var = d == NULL ? NULL : d->var;
+	dba_var var = dba_msg_find_by_id(msg, value);
 	const char* val = var == NULL ? NULL : dba_var_value(var);
 	if (val == NULL)
 		return -1;
@@ -584,8 +703,7 @@ inline static int get_date_value(dba_msg msg, int value)
 inline static double get_lat_value(dba_msg msg, int value)
 {
 	double res;
-	dba_msg_datum d = dba_msg_find_by_id(msg, value);
-	dba_var var = d == NULL ? NULL : d->var;
+	dba_var var = dba_msg_find_by_id(msg, value);
 	if (var == NULL || dba_var_value(var) == NULL)
 		return -1000000;
 	dba_var_enqd(var, &res);
@@ -597,11 +715,30 @@ dba_err filter_message(dba_rawmsg rmsg, bufrex_msg braw, dba_msgs msgs, void* da
 {
 	struct filter_data* fdata = (struct filter_data*)data;
 	double dval;
-	int msgs_reject = 1;
-	int i;
+	int msgs_reject;
+	int i, j, k;
 
 	if (msgs == NULL) return dba_error_ok();
 
+	for (i = 0; i < 20 && fdata->varfilter[i] != NULL; ++i)
+	{
+		dba_var test = fdata->varfilter[i];
+		int found = 0;
+		for (j = 0; !found && j < braw->subsets_count; ++j)
+		{
+			for (k = 0; !found && k < braw->subsets[j]->vars_count; ++k)
+			{
+				dba_var t = braw->subsets[j]->vars[k];
+				if (dba_var_code(test) == dba_var_code(t)
+				 && strcmp(dba_var_value(test), dba_var_value(t)) == 0)
+					found = 1;
+			}
+		}
+		if (!found)
+			return dba_error_ok();
+	}
+
+	msgs_reject = 1;
 	for (i = 0; i < msgs->len; ++i)
 	{
 		dba_msg msg = msgs->msgs[i];
@@ -681,6 +818,10 @@ dba_err do_filter(poptContext optCon)
 	dba_encoding otype;
 	struct filter_data fdata;
 	dba_record query;
+	memset(&fdata, 0, sizeof(fdata));
+	fdata.varfilter[0] = NULL;
+	dba_record_cursor c;
+	int i;
 
 	/* Throw away the command name */
 	poptGetArg(optCon);
@@ -696,6 +837,10 @@ dba_err do_filter(poptContext optCon)
 	fdata.lonmin = get_input_lat_value(query, DBA_KEY_LONMIN);
 	fdata.lonmax = get_input_lat_value(query, DBA_KEY_LONMAX);
 
+	for (i = 0, c = dba_record_iterate_first(query);
+			c != NULL && i < 20; c = dba_record_iterate_next(query, c), ++i)
+		fdata.varfilter[i] = dba_record_cursor_variable(c);
+
 	type = dba_cmdline_stringToMsgType(op_input_type, optCon);
 	otype = dba_cmdline_stringToMsgType(op_output_type, optCon);
 
@@ -988,6 +1133,10 @@ struct poptOption dbamsg_convert_options[] = {
 		"format of the input data ('bufr', 'crex', 'aof')", "type" },
 	{ "dest", 'd', POPT_ARG_STRING, &op_output_type, 0,
 		"format of the data in output ('bufr', 'crex', 'aof')", "type" },
+	{ "template", 0, POPT_ARG_STRING, &op_output_template, 0,
+		"template of the data in output (autoselect if not specified)", "type.sub.local" },
+	{ "report", 'r', POPT_ARG_STRING, &op_report, 0,
+		"force output data to be of this type of report", "rep_memo" },
 	{ NULL, 0, POPT_ARG_INCLUDE_TABLE, &grepTable, 0,
 		"Options used to filter messages" },
 	POPT_TABLEEND
@@ -1047,7 +1196,7 @@ static void init()
 	dbamsg.longdesc =
 		"Examine, dump and convert files containing meteorological data. "
 		"It supports observations encoded in BUFR, CREX and AOF formats";
-	dbamsg.ops = (struct op_dispatch_table*)calloc(10, sizeof(struct op_dispatch_table));
+	dbamsg.ops = (struct op_dispatch_table*)calloc(11, sizeof(struct op_dispatch_table));
 
 	dbamsg.ops[0].func = do_scan;
 	dbamsg.ops[0].aliases[0] = "scan";
@@ -1115,11 +1264,18 @@ static void init()
 	dbamsg.ops[8].longdesc = "Run testscript passing parts of filename on its stdin and checking the return code.  Then divide the input in half and try on each half.  Keep going until testscript does not fail in any portion of the file.  Output to stdout the smallest portion for which testscript fails.  This is useful to isolate the few messages in a file that cause problems";
 	dbamsg.ops[8].optable = dbamsg_bisect_options;
 
-	dbamsg.ops[9].func = NULL;
-	dbamsg.ops[9].usage = NULL;
-	dbamsg.ops[9].desc = NULL;
+	dbamsg.ops[9].func = do_head;
+	dbamsg.ops[9].aliases[0] = "head";
+	dbamsg.ops[9].usage = "head [options] filename [filename [...]]";
+	dbamsg.ops[9].desc = "Dump the contents of the header of a file with meteorological data";
 	dbamsg.ops[9].longdesc = NULL;
-	dbamsg.ops[9].optable = NULL;
+	dbamsg.ops[9].optable = dbamsg_scan_options;
+
+	dbamsg.ops[10].func = NULL;
+	dbamsg.ops[10].usage = NULL;
+	dbamsg.ops[10].desc = NULL;
+	dbamsg.ops[10].longdesc = NULL;
+	dbamsg.ops[10].optable = NULL;
 };
 
 static struct program_info proginfo = {
